/* error_handler.h -- Copyright (c) 2004 Michael Pyne <michael.pyne@kdemail.net>
 * This software is licensed under the terms of the GNU General Public License,
 * v2 (or any later version).  This software is distributed WITH ABSOLUTELY NO
 * WARRANTY!.
 */
#ifndef _ABAKUS_ERROR_HANDLER_H
#define _ABAKUS_ERROR_HANDLER_H

/* If compiling in C++ mode, we can use Qt.  Therefore, let's define our
 * custom event type, and predeclare some classes.
 */
#ifdef __cplusplus

#include <qevent.h>

class QObject;
typedef QObject * receiver_t;

class ErrorEvent : public QEvent
{
    public:
    ErrorEvent (const QString &str) : QEvent(User), m_error(str) { }
    QString message() const { return m_error; }

    private:
    QString m_error;
};

#else

typedef void * receiver_t;

#endif /* defined __cplusplus */

#ifdef __cplusplus
extern "C" {
#endif

/* C sucks, but that's no reason to keep us from using Qt.
 * This function can be called from the parser, and posts an error
 * event to the global qApp object, by creating a QString from str and
 * ref by using QString(str).arg(ref)
 *
 * This function won't know what receiver to use however, so you must
 * use declare_error_receiver() before using this function.
 */
void post_error_message(const char *str, const char *ref);

/* Use this function to declare a QObject to receive error event messages.
 */
void declare_error_receiver(receiver_t receiver);

#ifdef __cplusplus
};
#endif

#endif /* _ABAKUS_ERROR_HANDLER_H */
