/* function.h -- Copyright (c) 2004 Michael Pyne <michael.pyne@kdemail.net>
 * This software is licensed under the terms of the GNU General Public License,
 * v2 (or any later version).  This software is distributed WITH ABSOLUTELY NO
 * WARRANTY!.
 */

/* _FUNCTION_H was already taken, go figure. */
#ifndef _MPYNE_FUNCTION_H
#define _MPYNE_FUNCTION_H

#include <qobject.h>
#include <qstringlist.h>
#include <qstring.h>
#include <qmap.h>
#include <qdict.h>

class BaseFunction;

typedef long double number_t;

namespace Abakus
{
    typedef enum { Degrees, Radians } TrigMode;
}

struct UserFunction
{
    int sequenceNumber;
    BaseFunction *fn;
    QString varName;
};

struct Function {
    QString name;

    // A function is either builtin or user defined, this union is
    // used for both cases.
    union {
	number_t (*fn)(number_t); // Builtin
	UserFunction *userFn;     // User defined
    };

    bool returnsTrig;
    bool needsTrig;

    bool userDefined;
};

extern number_t parse_result;
extern int _lastParseStatusResult;

void setTrigMode(Abakus::TrigMode mode);
Abakus::TrigMode trigMode();

class FunctionManager : public QObject
{
    Q_OBJECT
    public:
    typedef QDict<Function> functionDict;

    // Definition of a function that maps one number value onto another number
    // of the same type.
    typedef number_t (*function_t)(const number_t);

    static FunctionManager *instance();

    Function *function(const QString &name);

    bool isFunction(const QString &name);
    bool isFunctionUserDefined(const QString &name);

    bool addFunction(BaseFunction *fn, const QString &dependantVar);
    bool addFunction(const QString &name, function_t fn);
    void removeFunction(const QString &name);

    typedef enum { Builtin, UserDefined, All } FunctionType;

    QStringList functionList(FunctionType type);

    signals:
    void signalFunctionAdded(const QString &name);
    void signalFunctionRemoved(const QString &name);

    private:
    FunctionManager(QObject *parent = 0, const char *name = "function manager");

    static FunctionManager *m_manager;
    functionDict m_dict;
};

int lastParseStatusResult(void);

number_t evaluateFunction(const Function *func, const number_t value);

/* Implemented in lexer.l due to prototype issues. */
number_t parseString(const char *str);

#endif
