#ifndef _ABAKUS_MAINWINDOW_H
#define _ABAKUS_MAINWINDOW_H

#include <kmainwindow.h>
#include "function.h"

class QVBox;
class QCheckBox;
class QRadioButton;
class QBoxLayout;
class QSplitter;

class KComboBox;
class KPopupMenu;
class KAction;
class KListView;
class KListBox;

// Main window class, handles events and layout and stuff
class MainWindow : public KMainWindow
{
    Q_OBJECT
    public:
    MainWindow();

    virtual bool event(QEvent *e);

    protected:
    virtual void contextMenuEvent(QContextMenuEvent *e);
    virtual bool eventFilter(QObject *o, QEvent *e);
    virtual bool queryExit();
    virtual void polish();

    private slots:
    void slotReturnPressed(const QString &text);
    void slotTextChanged(const QString &text);
    void slotEvaluate();

    void slotUpdateSize();

    void slotDegrees();
    void slotRadians();

    void slotEntrySelected(const QString &text);
    void slotResultSelected(const QString &text);

    void slotToggleMenuBar();
    void slotToggleFunctionList();
    void slotToggleVariableList();
    void slotToggleHistoryList();
    void slotToggleCompactMode();

    void slotNewValue(const QString &name, number_t value);
    void slotChangeValue(const QString &name, number_t value);
    void slotRemoveValue(const QString &name);

    void slotNewFunction(const QString &name);
    void slotRemoveFunction(const QString &name);

    private:
    int getParenthesesLevel(const QString &str);
    void loadConfig();
    void saveConfig();
    void setupLayout();
    void populateListViews();

    // Donated via JuK
    KAction *action(const char *key);

    template <class T> T *action(const char *key)
    {
	return dynamic_cast<T *>(action(key));
    }

    private:
    QVBox *m_history;
    QRadioButton *m_degrees;
    QRadioButton *m_radians;
    KComboBox *m_edit;
    KPopupMenu *m_popup;
    KListBox *m_result;
    QString m_lastError;
    QBoxLayout *m_layout;
    KListView *m_fnList, *m_varList;
    QSplitter *m_mainSplitter, *m_listSplitter;
    QSize m_newSize, m_oldSize;

    bool m_wasFnShown, m_wasVarShown, m_wasHistoryShown;
    bool m_compactMode;

    bool m_insert;
};

#endif
