#include <kdebug.h>
#include <kpopupmenu.h>
#include <klocale.h>

#include <qevent.h>
#include <qcursor.h>
#include <qsize.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qimage.h>
#include <qdragobject.h>

#include "resultlistbox.h"
#include "resultlistboxtext.h"

ResultListBox::ResultListBox(QWidget *parent, const char *name) :
    KListBox(parent, name), m_dragging(false)
{
    connect(this, SIGNAL(doubleClicked(QListBoxItem *, const QPoint &)),
                  SLOT(slotDoubleClicked(QListBoxItem *)));
}

void ResultListBox::mousePressEvent(QMouseEvent *e)
{
    KListBox::mousePressEvent(e);
    m_dragging = true;
}

void ResultListBox::mouseMoveEvent(QMouseEvent *e)
{
    if(m_dragging) {
	QListBoxItem *itemUnderCursor = itemAt(e->pos());
	ResultListBoxText *item = dynamic_cast<ResultListBoxText *>(itemUnderCursor);

	if(item) {
	    QString text = item->resultText();
	    if(e->pos().x() < width() / 2)
		text = item->expressionText();

	    QDragObject *drag = new QTextDrag(text, this);

	    QPixmap pixmap(makePixmap(text));

	    drag->setPixmap(pixmap);
	    drag->dragCopy();

	    m_dragging = false;
	}
    }
    else
	KListBox::mouseMoveEvent(e);
}

void ResultListBox::mouseReleaseEvent(QMouseEvent *e)
{
    m_dragging = false;
    KListBox::mouseReleaseEvent(e);
}

void ResultListBox::contextMenuEvent(QContextMenuEvent *e)
{
    static KPopupMenu *menu = 0;

    if(!menu)
	menu = constructPopupMenu();

    menu->popup(e->globalPos());
}

QPixmap ResultListBox::makePixmap(const QString &text)
{
    QColor background(234, 178, 230);

    int height = 2 * fontMetrics().height();
    QSize bonusSize (height, 0);
    QSize size(fontMetrics().width(text), height);
    QImage image(size + bonusSize, 32);

    image.setAlphaBuffer(false);
    image.fill(0); // All transparent pixels
    image.setAlphaBuffer(true);

    QPixmap pix(size + bonusSize);
    pix.fill(magenta); // Watch for incoming hacks

    QPainter painter(&pix);

    // Outline black, background white
    painter.setPen(black);
    painter.setBrush(background);

    // Stencil pill-shaped outline
    painter.drawRect(height / 2, 0, size.width() + 1, height);
    painter.drawPie(0, 0, height, height, 90 * 16, 180 * 16);
    painter.drawPie(size.width(), 0, height, height, 270 * 16, 180 * 16);

    // Erase interior lines
    painter.setPen(background);
    painter.drawRect(height / 2 - 1, 1, size.width() + 2, height - 2);

    // Draw text
    painter.setPen(black);
    painter.drawText(height / 2, height / 4, size.width(), size.height(), 0, text);

    QImage overlay(pix.convertToImage());

    // The images should have the same size, copy pixels from overlay to the
    // bottom unless the pixel is called magenta.

    for(int i = 0; i < image.width(); ++i)
	for(int j = 0; j < image.height(); ++j) {
	    if(QColor(overlay.pixel(i, j)) != magenta)
		image.setPixel(i, j, overlay.pixel(i, j));
	}

    pix.convertFromImage(image);
    return pix;
}

void ResultListBox::slotDoubleClicked(QListBoxItem *item)
{
    ResultListBoxText *textItem = dynamic_cast<ResultListBoxText *>(item);
    if(!textItem)
	return;

    QPoint hit = mapFromGlobal(QCursor::pos());
    if(hit.x() < width() / 2)
	emit signalEntrySelected(textItem->expressionText());
    else
	emit signalResultSelected(textItem->resultText());
}

KPopupMenu *ResultListBox::constructPopupMenu()
{
    KPopupMenu *menu = new KPopupMenu(this, "list box context menu");

    menu->insertItem(i18n("Clear &History"), this, SLOT(clear()), ALT+Key_R);

    return menu;
}

#include "resultlistbox.moc"
