#include <klistbox.h>
#include <klistview.h>

#include <qpainter.h>
#include <qfontmetrics.h>
#include <qfont.h>

#include "resultlistboxtext.h"

QColor ResultListBoxText::m_altColor = Qt::white;
bool ResultListBoxText::m_altColorCached = false;

ResultListBoxText::ResultListBoxText(KListBox *listBox,
                                     const QString &text,
				     const QString &result,
				     ResultListBoxText *after,
				     bool isError)
    : QListBoxText(listBox, text, after), m_text(text), m_result(result),
      m_alternate(false), m_wasError(isError)
{
    if(after)
	m_alternate = !after->drawAlternate();
}

template<class T>
T abakusMax(const T &l, const T &r)
{
    return (l > r) ? l : r;
}

int ResultListBoxText::width(const QListBox *lb) const
{
    int minWidth = 20; // 20 is the inter-text spacing plus the padding
    minWidth += lb->fontMetrics().width(m_text);

    // We draw the result in a bold font, be sure to compensate
    QFont f(lb->font());
    f.setBold(true);
    QFontMetrics fm(f);
    
    minWidth += fm.width(m_result);

    return abakusMax(minWidth, lb->width() - 2);
}

int ResultListBoxText::height(const QListBox *lb) const
{
    // Make the size slightly taller than normal
    return 5 * lb->fontMetrics().height() / 3;
}

void ResultListBoxText::paint(QPainter *p)
{
    QRect r(0, 0, listBox()->visibleWidth(), height(listBox()));
    QPixmap pix(r.width(), r.height());
    QPainter painter(&pix);
    
    painter.fillRect(r, drawAlternate() ? alternateColor() : listBox()->paletteBackgroundColor());

    // Add a margin
    r.addCoords(4, 0, -4, 0);

    painter.setPen(listBox()->paletteForegroundColor());
    painter.drawText(r, Qt::AlignLeft | Qt::AlignVCenter, m_text);

    if(m_wasError)
	painter.setPen(m_result == "OK" ? Qt::blue : Qt::red);

    QFont f(painter.font());
    f.setBold(true);
    painter.setFont(f);

    painter.drawText(r, Qt::AlignRight | Qt::AlignVCenter, m_result);
    p->drawPixmap(QPoint(0, 0), pix);
}

QColor ResultListBoxText::alternateColor()
{
    if(m_altColorCached)
	return m_altColor;

    // Use KListView's alternate background color
    KListView *temp = new KListView(0);
    m_altColor = temp->alternateBackground();
    delete temp;

    m_altColorCached = true;
    return m_altColor;
}

